@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2021	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -23,6 +23,7 @@
 
 #include <unistd.h>
 #include <arpa/inet.h>
+#include <time.h>
 #include "pool.h"
 #include "utils/elog.h"
 #include "context/pool_process_context.h"
@@ -104,10 +105,10 @@ pool_incremnet_local_session_id(void)
 /*
  * Return byte size of connection info(ConnectionInfo) on shmem.
  */
-size_t
+int
 pool_coninfo_size(void)
 {
-	size_t			size;
+	int			size;
 
 	size = pool_config->num_init_children *
 		pool_config->max_pool *
@@ -115,7 +116,7 @@ pool_coninfo_size(void)
 		sizeof(ConnectionInfo);
 
 	ereport(DEBUG1,
-			(errmsg("pool_coninfo_size: num_init_children (%d) * max_pool (%d) * MAX_NUM_BACKENDS (%d) * sizeof(ConnectionInfo) (%zu) = %zu bytes requested for shared memory",
+			(errmsg("pool_coninfo_size: num_init_children (%d) * max_pool (%d) * MAX_NUM_BACKENDS (%d) * sizeof(ConnectionInfo) (%zu) = %d bytes requested for shared memory",
 					pool_config->num_init_children,
 					pool_config->max_pool,
 					MAX_NUM_BACKENDS,
