@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -44,27 +44,37 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/time.h>
+#include <time.h>
+#include <limits.h>
 
 #ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
 
 #include "pool.h"
+#include "main/health_check.h"
+#include "main/pool_internal_comms.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
 #include "utils/elog.h"
+#include "utils/pool_ip.h"
+#include "utils/ps_status.h"
+#include "utils/pool_stream.h"
 
 #include "context/pool_process_context.h"
 #include "context/pool_session_context.h"
+#include "protocol/pool_pg_utils.h"
 #include "pool_config.h"
-#include "utils/pool_ip.h"
 #include "auth/md5.h"
 #include "auth/pool_hba.h"
-#include "utils/pool_stream.h"
+
+volatile POOL_HEALTH_CHECK_STATISTICS	*health_check_stats;	/* health check stats area in shared memory */
 
 static POOL_CONNECTION_POOL_SLOT * slot;
 static volatile sig_atomic_t reload_config_request = 0;
 static volatile sig_atomic_t restart_request = 0;
+volatile POOL_HEALTH_CHECK_STATISTICS *stats;
+
 static bool establish_persistent_connection(int node);
 static void discard_persistent_connection(int node);
 static RETSIGTYPE my_signal_handler(int sig);
@@ -72,7 +82,15 @@ static RETSIGTYPE reload_config_handler(int sig);
 static void reload_config(void);
 static RETSIGTYPE health_check_timer_handler(int sig);
 
+#ifdef HEALTHCHECK_OPTS
+#if HEALTHCHECK_OPTS > 0
+#define HEALTHCHECK_DEBUG
+#endif
+#endif
+
+#ifdef HEALTHCHECK_DEBUG
 static bool check_backend_down_request(int node, bool done_requests);
+#endif
 
 #undef CHECK_REQUEST
 #define CHECK_REQUEST \
@@ -103,6 +121,15 @@ do_health_check_child(int *node_id)
 	sigjmp_buf	local_sigjmp_buf;
 	MemoryContext HealthCheckMemoryContext;
 	char		psbuffer[NI_MAXHOST];
+	static struct timeval	start_time;
+	static struct timeval	end_time;
+	long	diff_t;
+
+	POOL_HEALTH_CHECK_STATISTICS	mystat;
+	stats = &health_check_stats[*node_id];
+
+	/* Set application name */
+	set_application_name_with_suffix(PT_HEALTH_CHECK, *node_id);
 
 	ereport(DEBUG1,
 			(errmsg("I am health check process pid:%d DB node id:%d", getpid(), *node_id)));
@@ -138,14 +165,6 @@ do_health_check_child(int *node_id)
 	/* Initialize per process context */
 	pool_init_process_context();
 
-	/*
-	 * Open pool_passwd.
-	 */
-	if (strcmp("", pool_config->pool_passwd))
-	{
-		pool_reopen_passwd_file();
-	}
-
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		pool_signal(SIGALRM, SIG_IGN);
@@ -157,21 +176,20 @@ do_health_check_child(int *node_id)
 	/* We can now handle ereport(ERROR) */
 	PG_exception_stack = &local_sigjmp_buf;
 
+	/* Initialize health check stats area */
+	stats->min_health_check_duration = INT_MAX;
 
 	for (;;)
 	{
 		MemoryContextSwitchTo(HealthCheckMemoryContext);
 		MemoryContextResetAndDeleteChildren(HealthCheckMemoryContext);
-		/*
-		 * Since HealthCheckMemoryContext is used for "slot", we need to clear it
-		 * so that new slot is allocated later on.
-		 */
-		slot = NULL;
+		bool	skipped = false;
 
 		CHECK_REQUEST;
 
 		if (pool_config->health_check_params[*node_id].health_check_period <= 0)
 		{
+			stats->min_health_check_duration = 0;
 			sleep(30);
 		}
 
@@ -184,10 +202,17 @@ do_health_check_child(int *node_id)
 			bool		result;
 			BackendInfo *bkinfo = pool_get_node_info(*node_id);
 
+			stats->total_count++;
+			gettimeofday(&start_time, NULL);
+
+			stats->last_health_check = time(NULL);
+
 			result = establish_persistent_connection(*node_id);
 
 			if (result && slot == NULL)
 			{
+				stats->last_failed_health_check = time(NULL);
+
 				if (POOL_DISALLOW_TO_FAILOVER(BACKEND_INFO(*node_id).flag))
 				{
 					ereport(LOG,
@@ -198,6 +223,8 @@ do_health_check_child(int *node_id)
 				{
 					bool		partial;
 
+					stats->fail_count++;
+
 					ereport(LOG, (errmsg("health check failed on node %d (timeout:%d)",
 										 *node_id, health_check_timer_expired)));
 
@@ -217,14 +244,57 @@ do_health_check_child(int *node_id)
 			}
 			else if (slot && bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == true)
 			{
+				stats->success_count++;
+				stats->last_successful_health_check = time(NULL);
+
 				/* The node has become reachable again. Reset
 				 * the quarantine state
 				 */
 				send_failback_request(*node_id, false, REQ_DETAIL_UPDATE | REQ_DETAIL_WATCHDOG);
 			}
+			else if (result && slot)
+			{
+				/* Health check succeeded */
+				stats->success_count++;
+				stats->last_successful_health_check = time(NULL);
+			}
+			else if (!result)
+			{
+				/* Health check skipped */
+				stats->skip_count++;
+				stats->last_skip_health_check = time(NULL);
+				skipped = true;
+			}
 
 			/* Discard persistent connections */
 			discard_persistent_connection(*node_id);
+
+			/*
+			 Update health check duration only if health check was not skipped
+			 since the duration could be very small (probably 0) if health
+			 check is skipped.
+			 */
+
+			if (!skipped)
+			{
+				gettimeofday(&end_time, NULL);
+
+				if (end_time.tv_sec > start_time.tv_sec)
+					diff_t = end_time.tv_usec - start_time.tv_usec + 1000000 * (end_time.tv_sec - start_time.tv_sec);
+				else
+					diff_t = end_time.tv_usec - start_time.tv_usec;
+
+				diff_t /= 1000;
+				stats->total_health_check_duration += diff_t;
+
+				if (diff_t > stats->max_health_check_duration)
+					stats->max_health_check_duration = diff_t;
+				if (diff_t < stats->min_health_check_duration)
+					stats->min_health_check_duration = diff_t;
+			}
+
+			memcpy(&mystat, (void *)stats, sizeof(mystat));
+
 			sleep(pool_config->health_check_params[*node_id].health_check_period);
 		}
 	}
@@ -250,13 +320,13 @@ establish_persistent_connection(int node)
 	/*
 	 * If the node is already in down status or unused, do nothing.
 	 * except when the node state is down because of quarantine operation
-	 * since we want to detect when the node comes back to life again to
+	 * since we want to detect when the node cames back to life again to
 	 * remove it from the quarantine state
 	 */
 	if (bkinfo->backend_status == CON_UNUSED ||
 		(bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == false))
 	{
-		/* get current time to use auto_failback_interval */
+		/* get current time to use auto_faliback_interval */
 		now = time(NULL);
 
 		if (pool_config->auto_failback && auto_failback_interval < now &&
@@ -283,11 +353,11 @@ establish_persistent_connection(int node)
 	 */
 	if (slot == NULL)
 	{
-		retry_cnt = pool_config->health_check_params[node].health_check_max_retries;
-
 		char	   *password = get_pgpool_config_user_password(pool_config->health_check_params[node].health_check_user,
 															   pool_config->health_check_params[node].health_check_password);
 
+		retry_cnt = pool_config->health_check_params[node].health_check_max_retries;
+
 		do
 		{
 			/*
@@ -316,18 +386,12 @@ establish_persistent_connection(int node)
 				CLEAR_ALARM;
 			}
 
-			/*
-			 * If health check test is enabled, check if fake down request is
-			 * set.  If set, discard_persistent_connection will set slot to
-			 * NULL.  This simulates a connection failure.
-			 */
-			if (pool_config->health_check_test)
+#ifdef HEALTHCHECK_DEBUG
+			if (slot && check_backend_down_request(node, false) == true)
 			{
-				if (slot && check_backend_down_request(node, false) == true)
-				{
-					discard_persistent_connection(node);
-				}
+				discard_persistent_connection(node);
 			}
+#endif
 
 			if (slot)
 			{
@@ -344,6 +408,8 @@ establish_persistent_connection(int node)
 
 			if (retry_cnt >= 0)
 			{
+				stats->retry_count++;
+
 				ereport(LOG,
 						(errmsg("health check retrying on DB node: %d (round:%d)",
 								node,
@@ -353,6 +419,19 @@ establish_persistent_connection(int node)
 			}
 		} while (retry_cnt >= 0);
 
+		/* Check if we need to refresh max retry count */
+
+		if (retry_cnt != pool_config->health_check_params[node].health_check_max_retries)
+		{
+			int ret_cnt;
+
+			retry_cnt++;
+			ret_cnt = pool_config->health_check_params[node].health_check_max_retries - retry_cnt;
+
+			if (ret_cnt > stats->max_retry_count)
+				stats->max_retry_count = ret_cnt;
+		}
+
 		if (password)
 			pfree(password);
 
@@ -360,13 +439,14 @@ establish_persistent_connection(int node)
 		{
 				ereport(LOG,
 					(errmsg("request auto failback, node id:%d", node)));
-				/* get current time to use auto_failback_interval */
+				/* get current time to use auto_faliback_interval */
 				now = time(NULL);
 				auto_failback_interval = now + pool_config->auto_failback_interval;
 
 				send_failback_request(node, true, REQ_DETAIL_CONFIRMED);
 		}
 	}
+
 	/* if check_failback is true, backend_status is DOWN or UNUSED. */
 	if (check_failback)
 	{
@@ -454,6 +534,38 @@ static RETSIGTYPE health_check_timer_handler(int sig)
 	errno = save_errno;
 }
 
+/*
+ * Returns the byte size of health check statistics area
+ */
+size_t
+health_check_stats_shared_memory_size(void)
+{
+	size_t	size;
+
+	size =  MAXALIGN(sizeof(POOL_HEALTH_CHECK_STATISTICS) * MAX_NUM_BACKENDS);
+	elog(LOG, "health_check_stats_shared_memory_size: requested size: %lu", size);
+	return size;
+}
+
+/*
+ * Initialize health check statistics area
+ */
+void
+health_check_stats_init(POOL_HEALTH_CHECK_STATISTICS *addr)
+{
+	int	i;
+
+	health_check_stats = addr;
+	memset((void *) health_check_stats, 0, health_check_stats_shared_memory_size());
+
+	for (i = 0 ;i < MAX_NUM_BACKENDS; i++)
+	{
+		health_check_stats[i].min_health_check_duration = INT_MAX;
+	}
+}
+
+#ifdef HEALTHCHECK_DEBUG
+
 /*
  * Node down request file. In the file, each line consists of "backend node
  * id", tab and "down".  If such a line found, check_backend_down_request()
@@ -465,7 +577,7 @@ static RETSIGTYPE health_check_timer_handler(int sig)
 /*
  * Check backend down request file with specified backend node id.  If it's
  * down ("down"), returns true and set the status to "already_down" to
- * prevent repeating failover. If it's other than "down", returns false.
+ * prevent repeatable * failover. If it's other than "down", returns false.
  *
  * When done_requests is true (second arg to function) the function returns
  * true if the node has already_done status in the file.
@@ -511,7 +623,7 @@ check_backend_down_request(int node, bool done_requests)
 		if (fgets(readbuf, MAXLINE - 1, fd) == 0)
 			break;
 
-		strncpy(buf, readbuf, sizeof(buf) - 1);
+		strncpy(buf, readbuf, sizeof(buf));
 		if (strlen(readbuf) > 0 && readbuf[strlen(readbuf) - 1] == '\n')
 			buf[strlen(readbuf) - 1] = '\0';
 		sscanf(buf, "%d\t%s", &node_id, status);
@@ -553,11 +665,32 @@ check_backend_down_request(int node, bool done_requests)
 
 	fclose(fd);
 
-	if (writebuf)
-		free(writebuf);
-
 	if (!found)
 		return false;
 
+#ifdef NOT_USED
+	fd = fopen(backend_down_request_file, "w");
+	if (!fd)
+	{
+		ereport(WARNING,
+				(errmsg("check_backend_down_request: failed to open file for writing %s",
+						backend_down_request_file),
+				 errdetail("%m")));
+		return false;
+	}
+
+	if (fwrite(writebuf, 1, strlen(writebuf), fd) != strlen(writebuf))
+	{
+		ereport(WARNING,
+				(errmsg("check_backend_down_request: failed to write %s",
+						backend_down_request_file),
+				 errdetail("%m")));
+		fclose(fd);
+		return false;
+	}
+	fclose(fd);
+#endif
+
 	return true;
 }
+#endif
