@@ -1,8 +1,9 @@
 /*
- * Copyright (c) 2003-2015, PgPool Global Development Group
+ * Copyright (c) 2003-2019, PgPool Global Development Group
  * Copyright (c) 1983, 1995, 1996 Eric P. Allman
  * Copyright (c) 1988, 1993
  *	The Regents of the University of California.  All rights reserved.
+ * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,22 +36,6 @@
 #include "c.h"
 #endif
 
-#include <ctype.h>
-#ifdef _MSC_VER
-#include <float.h>				/* for _isnan */
-#endif
-#include <limits.h>
-#include <math.h>
-#ifndef WIN32
-#include <sys/ioctl.h>
-#endif
-#include <sys/param.h>
-
-
-#ifndef NL_ARGMAX
-#define NL_ARGMAX 16
-#endif
-
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -62,6 +47,14 @@
 #include "stringinfo.h"
 #include "utils/palloc.h"
 
+/*
+ * We used to use the platform's NL_ARGMAX here, but that's a bad idea,
+ * first because the point of this module is to remove platform dependencies
+ * not perpetuate them, and second because some platforms use ridiculously
+ * large values, leading to excessive stack consumption in dopr().
+ */
+#define PG_NL_ARGMAX 31
+
 /* --------------------------------------------------------------------- */
 /* c.h                                                                   */
 /* --------------------------------------------------------------------- */
@@ -112,8 +105,8 @@
  *	SNPRINTF, VSNPRINTF and friends
  *
  * These versions have been grabbed off the net.  They have been
- * cleaned up to compile properly and support for most of the Single Unix
- * Specification has been added.  Remaining unimplemented features are:
+ * cleaned up to compile properly and support for most of the C99
+ * specification has been added.  Remaining unimplemented features are:
  *
  * 1. No locale support: the radix character is always '.' and the '
  * (single quote) format flag is ignored.
@@ -126,26 +119,33 @@
  *
  * 5. Space and '#' flags are not implemented.
  *
+ * In addition, we support some extensions over C99:
  *
- * The result values of these functions are not the same across different
- * platforms.  This implementation is compatible with the Single Unix Spec:
+ * 1. Argument order control through "%n$" and "*n$", as required by POSIX.
  *
- * 1. -1 is returned only if processing is abandoned due to an invalid
- * parameter, such as incorrect format string.  (Although not required by
- * the spec, this happens only when no characters have yet been transmitted
- * to the destination.)
+ * 2. "%m" expands to the value of strerror(errno), where errno is the
+ * value that variable had at the start of the call.  This is a glibc
+ * extension, but a very useful one.
  *
- * 2. For snprintf and sprintf, 0 is returned if str == NULL or count == 0;
- * no data has been stored.
  *
- * 3. Otherwise, the number of bytes actually transmitted to the destination
- * is returned (excluding the trailing '\0' for snprintf and sprintf).
+ * Historically the result values of sprintf/snprintf varied across platforms.
+ * This implementation now follows the C99 standard:
  *
- * For snprintf with nonzero count, the result cannot be more than count-1
- * (a trailing '\0' is always stored); it is not possible to distinguish
- * buffer overrun from exact fit.  This is unlike some implementations that
- * return the number of bytes that would have been needed for the complete
- * result string.
+ * 1. -1 is returned if an error is detected in the format string, or if
+ * a write to the target stream fails (as reported by fwrite).  Note that
+ * overrunning snprintf's target buffer is *not* an error.
+ *
+ * 2. For successful writes to streams, the actual number of bytes written
+ * to the stream is returned.
+ *
+ * 3. For successful sprintf/snprintf, the number of bytes that would have
+ * been written to an infinite-size buffer (excluding the trailing '\0')
+ * is returned.  snprintf will truncate its output to fit in the buffer
+ * (ensuring a trailing '\0' unless count == 0), but this is not reflected
+ * in the function result.
+ *
+ * snprintf buffer overrun can be detected by checking for function result
+ * greater than or equal to the supplied count.
  */
 
 /**************************************************************
@@ -159,20 +159,34 @@
 /* Prevent recursion */
 #undef	vsnprintf
 #undef	snprintf
+#undef	vsprintf
 #undef	sprintf
 #undef	vfprintf
 #undef	fprintf
+#undef	vprintf
 #undef	printf
 
-/* Info about where the formatted output is going */
+/*
+ * Info about where the formatted output is going.
+ *
+ * dopr and subroutines will not write at/past bufend, but snprintf
+ * reserves one byte, ensuring it may place the trailing '\0' there.
+ *
+ * In snprintf, we use nchars to count the number of bytes dropped on the
+ * floor due to buffer overrun.  The correct result of snprintf is thus
+ * (bufptr - bufstart) + nchars.  (This isn't as inconsistent as it might
+ * seem: nchars is the number of emitted bytes that are not in the buffer now,
+ * either because we sent them to the stream or because we couldn't fit them
+ * into the buffer to begin with.)
+ */
 typedef struct
 {
 	char	   *bufptr;			/* next buffer output position */
 	char	   *bufstart;		/* first buffer element */
-	char	   *bufend;			/* last buffer element, or NULL */
+	char	   *bufend;			/* last+1 buffer element, or NULL */
 	/* bufend == NULL is for sprintf, where we assume buf is big enough */
 	FILE	   *stream;			/* eventual output destination, or NULL */
-	int			nchars;			/* # chars already sent to stream */
+	int			nchars;			/* # chars sent to stream, or dropped */
 	bool		failed;			/* call is a failure; errno is set */
 } PrintfTarget;
 
@@ -196,7 +210,7 @@ typedef union
 {
 	int			i;
 	long		l;
-	int64		ll;
+	long long	ll;
 	double		d;
 	char	   *cptr;
 } PrintfArgValue;
@@ -206,21 +220,39 @@ static void flushbuffer(PrintfTarget *target);
 static void dopr(PrintfTarget *target, const char *format, va_list args);
 
 
+/*
+ * Externally visible entry points.
+ *
+ * All of these are just wrappers around dopr().  Note it's essential that
+ * they not change the value of "errno" before reaching dopr().
+ */
+
 int
 pg_vsnprintf(char *str, size_t count, const char *fmt, va_list args)
 {
 	PrintfTarget target;
+	char		onebyte[1];
 
-	if (str == NULL || count == 0)
-		return 0;
+	/*
+	 * C99 allows the case str == NULL when count == 0.  Rather than
+	 * special-casing this situation further down, we substitute a one-byte
+	 * local buffer.  Callers cannot tell, since the function result doesn't
+	 * depend on count.
+	 */
+	if (count == 0)
+	{
+		str = onebyte;
+		count = 1;
+	}
 	target.bufstart = target.bufptr = str;
 	target.bufend = str + count - 1;
 	target.stream = NULL;
-	/* target.nchars is unused in this case */
+	target.nchars = 0;
 	target.failed = false;
 	dopr(&target, fmt, args);
 	*(target.bufptr) = '\0';
-	return target.failed ? -1 : (target.bufptr - target.bufstart);
+	return target.failed ? -1 : (target.bufptr - target.bufstart
+								 + target.nchars);
 }
 
 #if 0
@@ -236,21 +268,20 @@ pg_snprintf(char *str, size_t count, const char *fmt,...)
 	return len;
 }
 
-static int
+int
 pg_vsprintf(char *str, const char *fmt, va_list args)
 {
 	PrintfTarget target;
 
-	if (str == NULL)
-		return 0;
 	target.bufstart = target.bufptr = str;
 	target.bufend = NULL;
 	target.stream = NULL;
-	/* target.nchars is unused in this case */
+	target.nchars = 0;			/* not really used in this case */
 	target.failed = false;
 	dopr(&target, fmt, args);
 	*(target.bufptr) = '\0';
-	return target.failed ? -1 : (target.bufptr - target.bufstart);
+	return target.failed ? -1 : (target.bufptr - target.bufstart
+								 + target.nchars);
 }
 
 int
@@ -277,7 +308,7 @@ pg_vfprintf(FILE *stream, const char *fmt, va_list args)
 		return -1;
 	}
 	target.bufstart = target.bufptr = buffer;
-	target.bufend = buffer + sizeof(buffer) - 1;
+	target.bufend = buffer + sizeof(buffer);	/* use the whole buffer */
 	target.stream = stream;
 	target.nchars = 0;
 	target.failed = false;
@@ -299,6 +330,12 @@ pg_fprintf(FILE *stream, const char *fmt,...)
 	return len;
 }
 
+int
+pg_vprintf(const char *fmt, va_list args)
+{
+	return pg_vfprintf(stdout, fmt, args);
+}
+
 int
 pg_printf(const char *fmt,...)
 {
@@ -321,6 +358,10 @@ flushbuffer(PrintfTarget *target)
 {
 	size_t		nc = target->bufptr - target->bufstart;
 
+	/*
+	 * Don't write anything if we already failed; this is to ensure we
+	 * preserve the original failure's errno.
+	 */
 	if (!target->failed && nc > 0)
 	{
 		size_t		written;
@@ -334,35 +375,69 @@ flushbuffer(PrintfTarget *target)
 }
 
 
-static void fmtstr(char *value, int leftjust, int minlen, int maxwidth,
-	   int pointflag, PrintfTarget *target);
+static bool find_arguments(const char *format, va_list args,
+						   PrintfArgValue *argvalues);
+static void fmtstr(const char *value, int leftjust, int minlen, int maxwidth,
+				   int pointflag, PrintfTarget *target);
 static void fmtptr(void *value, PrintfTarget *target);
-static void fmtint(int64 value, char type, int forcesign,
-	   int leftjust, int minlen, int zpad, int precision, int pointflag,
-	   PrintfTarget *target);
+static void fmtint(long long value, char type, int forcesign,
+				   int leftjust, int minlen, int zpad, int precision, int pointflag,
+				   PrintfTarget *target);
 static void fmtchar(int value, int leftjust, int minlen, PrintfTarget *target);
 static void fmtfloat(double value, char type, int forcesign,
-		 int leftjust, int minlen, int zpad, int precision, int pointflag,
-		 PrintfTarget *target);
+					 int leftjust, int minlen, int zpad, int precision, int pointflag,
+					 PrintfTarget *target);
 static void dostr(const char *str, int slen, PrintfTarget *target);
 static void dopr_outch(int c, PrintfTarget *target);
+static void dopr_outchmulti(int c, int slen, PrintfTarget *target);
 static int	adjust_sign(int is_negative, int forcesign, int *signvalue);
-static void adjust_padlen(int minlen, int vallen, int leftjust, int *padlen);
-static void leading_pad(int zpad, int *signvalue, int *padlen,
-			PrintfTarget *target);
-static void trailing_pad(int *padlen, PrintfTarget *target);
+static int	compute_padlen(int minlen, int vallen, int leftjust);
+static void leading_pad(int zpad, int signvalue, int *padlen,
+						PrintfTarget *target);
+static void trailing_pad(int padlen, PrintfTarget *target);
+
+/*
+ * If strchrnul exists (it's a glibc-ism), it's a good bit faster than the
+ * equivalent manual loop.  If it doesn't exist, provide a replacement.
+ *
+ * Note: glibc declares this as returning "char *", but that would require
+ * casting away const internally, so we don't follow that detail.
+ */
+#ifndef HAVE_STRCHRNUL
+
+static inline const char *
+strchrnul(const char *s, int c)
+{
+	while (*s != '\0' && *s != c)
+		s++;
+	return s;
+}
+
+#else
+
+/*
+ * glibc's <string.h> declares strchrnul only if _GNU_SOURCE is defined.
+ * While we typically use that on glibc platforms, configure will set
+ * HAVE_STRCHRNUL whether it's used or not.  Fill in the missing declaration
+ * so that this file will compile cleanly with or without _GNU_SOURCE.
+ */
+#ifndef _GNU_SOURCE
+extern char *strchrnul(const char *s, int c);
+#endif
+
+#endif							/* HAVE_STRCHRNUL */
 
 
 /*
- * dopr(): poor man's version of doprintf
+ * dopr(): the guts of *printf for all cases.
  */
 static void
 dopr(PrintfTarget *target, const char *format, va_list args)
 {
-	const char *format_start = format;
+	int			save_errno = errno;
+	const char *first_pct = NULL;
 	int			ch;
 	bool		have_dollar;
-	bool		have_non_dollar;
 	bool		have_star;
 	bool		afterstar;
 	int			accum;
@@ -374,233 +449,68 @@ dopr(PrintfTarget *target, const char *format, va_list args)
 	int			precision;
 	int			zpad;
 	int			forcesign;
-	int			last_dollar;
 	int			fmtpos;
 	int			cvalue;
-	int64		numvalue;
+	long long	numvalue;
 	double		fvalue;
 	char	   *strvalue;
-	int			i;
-	PrintfArgType argtypes[NL_ARGMAX + 1];
-	PrintfArgValue argvalues[NL_ARGMAX + 1];
+	PrintfArgValue argvalues[PG_NL_ARGMAX + 1];
 
 	/*
-	 * Parse the format string to determine whether there are %n$ format
-	 * specs, and identify the types and order of the format parameters.
+	 * Initially, we suppose the format string does not use %n$.  The first
+	 * time we come to a conversion spec that has that, we'll call
+	 * find_arguments() to check for consistent use of %n$ and fill the
+	 * argvalues array with the argument values in the correct order.
 	 */
-	have_dollar = have_non_dollar = false;
-	last_dollar = 0;
-	MemSet(argtypes, 0, sizeof(argtypes));
+	have_dollar = false;
 
-	while ((ch = *format++) != '\0')
+	while (*format != '\0')
 	{
-		if (ch != '%')
-			continue;
-		longflag = longlongflag = pointflag = 0;
-		fmtpos = accum = 0;
-		afterstar = false;
-nextch1:
-		ch = *format++;
-		if (ch == '\0')
-			break;				/* illegal, but we don't complain */
-		switch (ch)
+		/* Locate next conversion specifier */
+		if (*format != '%')
 		{
-			case '-':
-			case '+':
-				goto nextch1;
-			case '0':
-			case '1':
-			case '2':
-			case '3':
-			case '4':
-			case '5':
-			case '6':
-			case '7':
-			case '8':
-			case '9':
-				accum = accum * 10 + (ch - '0');
-				goto nextch1;
-			case '.':
-				pointflag = 1;
-				accum = 0;
-				goto nextch1;
-			case '*':
-				if (afterstar)
-					have_non_dollar = true; /* multiple stars */
-				afterstar = true;
-				accum = 0;
-				goto nextch1;
-			case '$':
-				have_dollar = true;
-				if (accum <= 0 || accum > NL_ARGMAX)
-					goto bad_format;
-				if (afterstar)
-				{
-					if (argtypes[accum] &&
-						argtypes[accum] != ATYPE_INT)
-						goto bad_format;
-					argtypes[accum] = ATYPE_INT;
-					last_dollar = Max(last_dollar, accum);
-					afterstar = false;
-				}
-				else
-					fmtpos = accum;
-				accum = 0;
-				goto nextch1;
-			case 'l':
-				if (longflag)
-					longlongflag = 1;
-				else
-					longflag = 1;
-				goto nextch1;
-			case 'z':
-#if SIZEOF_SIZE_T == 8
-#ifdef HAVE_LONG_INT_64
-				longflag = 1;
-#elif defined(HAVE_LONG_LONG_INT_64)
-				longlongflag = 1;
-#else
-#error "Don't know how to print 64bit integers"
-#endif
-#else
-				/* assume size_t is same size as int */
-#endif
-				goto nextch1;
-			case 'h':
-			case '\'':
-				/* ignore these */
-				goto nextch1;
-			case 'd':
-			case 'i':
-			case 'o':
-			case 'u':
-			case 'x':
-			case 'X':
-				if (fmtpos)
-				{
-					PrintfArgType atype;
+			/* Scan to next '%' or end of string */
+			const char *next_pct = strchrnul(format + 1, '%');
 
-					if (longlongflag)
-						atype = ATYPE_LONGLONG;
-					else if (longflag)
-						atype = ATYPE_LONG;
-					else
-						atype = ATYPE_INT;
-					if (argtypes[fmtpos] &&
-						argtypes[fmtpos] != atype)
-						goto bad_format;
-					argtypes[fmtpos] = atype;
-					last_dollar = Max(last_dollar, fmtpos);
-				}
-				else
-					have_non_dollar = true;
+			/* Dump literal data we just scanned over */
+			dostr(format, next_pct - format, target);
+			if (target->failed)
 				break;
-			case 'c':
-				if (fmtpos)
-				{
-					if (argtypes[fmtpos] &&
-						argtypes[fmtpos] != ATYPE_INT)
-						goto bad_format;
-					argtypes[fmtpos] = ATYPE_INT;
-					last_dollar = Max(last_dollar, fmtpos);
-				}
-				else
-					have_non_dollar = true;
-				break;
-			case 's':
-			case 'p':
-				if (fmtpos)
-				{
-					if (argtypes[fmtpos] &&
-						argtypes[fmtpos] != ATYPE_CHARPTR)
-						goto bad_format;
-					argtypes[fmtpos] = ATYPE_CHARPTR;
-					last_dollar = Max(last_dollar, fmtpos);
-				}
-				else
-					have_non_dollar = true;
-				break;
-			case 'e':
-			case 'E':
-			case 'f':
-			case 'g':
-			case 'G':
-				if (fmtpos)
-				{
-					if (argtypes[fmtpos] &&
-						argtypes[fmtpos] != ATYPE_DOUBLE)
-						goto bad_format;
-					argtypes[fmtpos] = ATYPE_DOUBLE;
-					last_dollar = Max(last_dollar, fmtpos);
-				}
-				else
-					have_non_dollar = true;
-				break;
-			case '%':
+
+			if (*next_pct == '\0')
 				break;
+			format = next_pct;
 		}
 
 		/*
-		 * If we finish the spec with afterstar still set, there's a
-		 * non-dollar star in there.
+		 * Remember start of first conversion spec; if we find %n$, then it's
+		 * sufficient for find_arguments() to start here, without rescanning
+		 * earlier literal text.
 		 */
-		if (afterstar)
-			have_non_dollar = true;
-	}
+		if (first_pct == NULL)
+			first_pct = format;
 
-	/* Per spec, you use either all dollar or all not. */
-	if (have_dollar && have_non_dollar)
-		goto bad_format;
+		/* Process conversion spec starting at *format */
+		format++;
 
-	/*
-	 * In dollar mode, collect the arguments in physical order.
-	 */
-	for (i = 1; i <= last_dollar; i++)
-	{
-		switch (argtypes[i])
+		/* Fast path for conversion spec that is exactly %s */
+		if (*format == 's')
 		{
-			case ATYPE_NONE:
-				goto bad_format;
-			case ATYPE_INT:
-				argvalues[i].i = va_arg(args, int);
+			format++;
+			strvalue = va_arg(args, char *);
+			Assert(strvalue != NULL);
+			dostr(strvalue, strlen(strvalue), target);
+			if (target->failed)
 				break;
-			case ATYPE_LONG:
-				argvalues[i].l = va_arg(args, long);
-				break;
-			case ATYPE_LONGLONG:
-				argvalues[i].ll = va_arg(args, int64);
-				break;
-			case ATYPE_DOUBLE:
-				argvalues[i].d = va_arg(args, double);
-				break;
-			case ATYPE_CHARPTR:
-				argvalues[i].cptr = va_arg(args, char *);
-				break;
-		}
-	}
-
-	/*
-	 * At last we can parse the format for real.
-	 */
-	format = format_start;
-	while ((ch = *format++) != '\0')
-	{
-		if (target->failed)
-			break;
-
-		if (ch != '%')
-		{
-			dopr_outch(ch, target);
 			continue;
 		}
+
 		fieldwidth = precision = zpad = leftjust = forcesign = 0;
 		longflag = longlongflag = pointflag = 0;
 		fmtpos = accum = 0;
 		have_star = afterstar = false;
 nextch2:
 		ch = *format++;
-		if (ch == '\0')
-			break;				/* illegal, but we don't complain */
 		switch (ch)
 		{
 			case '-':
@@ -636,7 +546,11 @@ nextch2:
 			case '*':
 				if (have_dollar)
 				{
-					/* process value after reading n$ */
+					/*
+					 * We'll process value after reading n$.  Note it's OK to
+					 * assume have_dollar is set correctly, because in a valid
+					 * format string the initial % must have had n$ if * does.
+					 */
 					afterstar = true;
 				}
 				else
@@ -667,6 +581,14 @@ nextch2:
 				accum = 0;
 				goto nextch2;
 			case '$':
+				/* First dollar sign? */
+				if (!have_dollar)
+				{
+					/* Yup, so examine all conversion specs in format */
+					if (!find_arguments(first_pct, args, argvalues))
+						goto bad_format;
+					have_dollar = true;
+				}
 				if (afterstar)
 				{
 					/* fetch and process star value */
@@ -740,7 +662,7 @@ nextch2:
 				else
 				{
 					if (longlongflag)
-						numvalue = va_arg(args, int64);
+						numvalue = va_arg(args, long long);
 					else if (longflag)
 						numvalue = va_arg(args, long);
 					else
@@ -763,7 +685,7 @@ nextch2:
 				if (have_dollar)
 				{
 					if (longlongflag)
-						numvalue = (uint64) argvalues[fmtpos].ll;
+						numvalue = (unsigned long long) argvalues[fmtpos].ll;
 					else if (longflag)
 						numvalue = (unsigned long) argvalues[fmtpos].l;
 					else
@@ -772,7 +694,7 @@ nextch2:
 				else
 				{
 					if (longlongflag)
-						numvalue = (uint64) va_arg(args, int64);
+						numvalue = (unsigned long long) va_arg(args, long long);
 					else if (longflag)
 						numvalue = (unsigned long) va_arg(args, long);
 					else
@@ -807,6 +729,8 @@ nextch2:
 					strvalue = argvalues[fmtpos].cptr;
 				else
 					strvalue = va_arg(args, char *);
+				/* Whine if someone tries to print a NULL string */
+				Assert(strvalue != NULL);
 				fmtstr(strvalue, leftjust, fieldwidth, precision, pointflag,
 					   target);
 				break;
@@ -839,10 +763,30 @@ nextch2:
 						 precision, pointflag,
 						 target);
 				break;
+			case 'm':
+				{
+					char		errbuf[PG_STRERROR_R_BUFLEN];
+					const char *errm = strerror_r(save_errno,
+												  errbuf, sizeof(errbuf));
+
+					dostr(errm, strlen(errm), target);
+				}
+				break;
 			case '%':
 				dopr_outch('%', target);
 				break;
+			default:
+
+				/*
+				 * Anything else --- in particular, '\0' indicating end of
+				 * format string --- is bogus.
+				 */
+				goto bad_format;
 		}
+
+		/* Check for failure after each conversion spec */
+		if (target->failed)
+			break;
 	}
 
 	return;
@@ -852,18 +796,236 @@ bad_format:
 	target->failed = true;
 }
 
-static size_t
-pg_strnlen(const char *str, size_t maxlen)
+/*
+ * find_arguments(): sort out the arguments for a format spec with %n$
+ *
+ * If format is valid, return true and fill argvalues[i] with the value
+ * for the conversion spec that has %i$ or *i$.  Else return false.
+ */
+static bool
+find_arguments(const char *format, va_list args,
+			   PrintfArgValue *argvalues)
 {
-	const char *p = str;
+	int			ch;
+	bool		afterstar;
+	int			accum;
+	int			longlongflag;
+	int			longflag;
+	int			fmtpos;
+	int			i;
+	int			last_dollar;
+	PrintfArgType argtypes[PG_NL_ARGMAX + 1];
 
-	while (maxlen-- > 0 && *p)
-		p++;
-	return p - str;
+	/* Initialize to "no dollar arguments known" */
+	last_dollar = 0;
+	MemSet(argtypes, 0, sizeof(argtypes));
+
+	/*
+	 * This loop must accept the same format strings as the one in dopr().
+	 * However, we don't need to analyze them to the same level of detail.
+	 *
+	 * Since we're only called if there's a dollar-type spec somewhere, we can
+	 * fail immediately if we find a non-dollar spec.  Per the C99 standard,
+	 * all argument references in the format string must be one or the other.
+	 */
+	while (*format != '\0')
+	{
+		/* Locate next conversion specifier */
+		if (*format != '%')
+		{
+			/* Unlike dopr, we can just quit if there's no more specifiers */
+			format = strchr(format + 1, '%');
+			if (format == NULL)
+				break;
+		}
+
+		/* Process conversion spec starting at *format */
+		format++;
+		longflag = longlongflag = 0;
+		fmtpos = accum = 0;
+		afterstar = false;
+nextch1:
+		ch = *format++;
+		switch (ch)
+		{
+			case '-':
+			case '+':
+				goto nextch1;
+			case '0':
+			case '1':
+			case '2':
+			case '3':
+			case '4':
+			case '5':
+			case '6':
+			case '7':
+			case '8':
+			case '9':
+				accum = accum * 10 + (ch - '0');
+				goto nextch1;
+			case '.':
+				accum = 0;
+				goto nextch1;
+			case '*':
+				if (afterstar)
+					return false;	/* previous star missing dollar */
+				afterstar = true;
+				accum = 0;
+				goto nextch1;
+			case '$':
+				if (accum <= 0 || accum > PG_NL_ARGMAX)
+					return false;
+				if (afterstar)
+				{
+					if (argtypes[accum] &&
+						argtypes[accum] != ATYPE_INT)
+						return false;
+					argtypes[accum] = ATYPE_INT;
+					last_dollar = Max(last_dollar, accum);
+					afterstar = false;
+				}
+				else
+					fmtpos = accum;
+				accum = 0;
+				goto nextch1;
+			case 'l':
+				if (longflag)
+					longlongflag = 1;
+				else
+					longflag = 1;
+				goto nextch1;
+			case 'z':
+#if SIZEOF_SIZE_T == 8
+#ifdef HAVE_LONG_INT_64
+				longflag = 1;
+#elif defined(HAVE_LONG_LONG_INT_64)
+				longlongflag = 1;
+#else
+#error "Don't know how to print 64bit integers"
+#endif
+#else
+				/* assume size_t is same size as int */
+#endif
+				goto nextch1;
+			case 'h':
+			case '\'':
+				/* ignore these */
+				goto nextch1;
+			case 'd':
+			case 'i':
+			case 'o':
+			case 'u':
+			case 'x':
+			case 'X':
+				if (fmtpos)
+				{
+					PrintfArgType atype;
+
+					if (longlongflag)
+						atype = ATYPE_LONGLONG;
+					else if (longflag)
+						atype = ATYPE_LONG;
+					else
+						atype = ATYPE_INT;
+					if (argtypes[fmtpos] &&
+						argtypes[fmtpos] != atype)
+						return false;
+					argtypes[fmtpos] = atype;
+					last_dollar = Max(last_dollar, fmtpos);
+				}
+				else
+					return false;	/* non-dollar conversion spec */
+				break;
+			case 'c':
+				if (fmtpos)
+				{
+					if (argtypes[fmtpos] &&
+						argtypes[fmtpos] != ATYPE_INT)
+						return false;
+					argtypes[fmtpos] = ATYPE_INT;
+					last_dollar = Max(last_dollar, fmtpos);
+				}
+				else
+					return false;	/* non-dollar conversion spec */
+				break;
+			case 's':
+			case 'p':
+				if (fmtpos)
+				{
+					if (argtypes[fmtpos] &&
+						argtypes[fmtpos] != ATYPE_CHARPTR)
+						return false;
+					argtypes[fmtpos] = ATYPE_CHARPTR;
+					last_dollar = Max(last_dollar, fmtpos);
+				}
+				else
+					return false;	/* non-dollar conversion spec */
+				break;
+			case 'e':
+			case 'E':
+			case 'f':
+			case 'g':
+			case 'G':
+				if (fmtpos)
+				{
+					if (argtypes[fmtpos] &&
+						argtypes[fmtpos] != ATYPE_DOUBLE)
+						return false;
+					argtypes[fmtpos] = ATYPE_DOUBLE;
+					last_dollar = Max(last_dollar, fmtpos);
+				}
+				else
+					return false;	/* non-dollar conversion spec */
+				break;
+			case 'm':
+			case '%':
+				break;
+			default:
+				return false;	/* bogus format string */
+		}
+
+		/*
+		 * If we finish the spec with afterstar still set, there's a
+		 * non-dollar star in there.
+		 */
+		if (afterstar)
+			return false;		/* non-dollar conversion spec */
+	}
+
+	/*
+	 * Format appears valid so far, so collect the arguments in physical
+	 * order.  (Since we rejected any non-dollar specs that would have
+	 * collected arguments, we know that dopr() hasn't collected any yet.)
+	 */
+	for (i = 1; i <= last_dollar; i++)
+	{
+		switch (argtypes[i])
+		{
+			case ATYPE_NONE:
+				return false;
+			case ATYPE_INT:
+				argvalues[i].i = va_arg(args, int);
+				break;
+			case ATYPE_LONG:
+				argvalues[i].l = va_arg(args, long);
+				break;
+			case ATYPE_LONGLONG:
+				argvalues[i].ll = va_arg(args, long long);
+				break;
+			case ATYPE_DOUBLE:
+				argvalues[i].d = va_arg(args, double);
+				break;
+			case ATYPE_CHARPTR:
+				argvalues[i].cptr = va_arg(args, char *);
+				break;
+		}
+	}
+
+	return true;
 }
 
 static void
-fmtstr(char *value, int leftjust, int minlen, int maxwidth,
+fmtstr(const char *value, int leftjust, int minlen, int maxwidth,
 	   int pointflag, PrintfTarget *target)
 {
 	int			padlen,
@@ -874,21 +1036,21 @@ fmtstr(char *value, int leftjust, int minlen, int maxwidth,
 	 * than that.
 	 */
 	if (pointflag)
-		vallen = pg_strnlen(value, maxwidth);
+		vallen = strnlen(value, maxwidth);
 	else
 		vallen = strlen(value);
 
-	adjust_padlen(minlen, vallen, leftjust, &padlen);
+	padlen = compute_padlen(minlen, vallen, leftjust);
 
-	while (padlen > 0)
+	if (padlen > 0)
 	{
-		dopr_outch(' ', target);
-		--padlen;
+		dopr_outchmulti(' ', padlen, target);
+		padlen = 0;
 	}
 
 	dostr(value, vallen, target);
 
-	trailing_pad(&padlen, target);
+	trailing_pad(padlen, target);
 }
 
 static void
@@ -906,17 +1068,18 @@ fmtptr(void *value, PrintfTarget *target)
 }
 
 static void
-fmtint(int64 value, char type, int forcesign, int leftjust,
+fmtint(long long value, char type, int forcesign, int leftjust,
 	   int minlen, int zpad, int precision, int pointflag,
 	   PrintfTarget *target)
 {
-	uint64		base;
+	unsigned long long base;
+	unsigned long long uvalue;
 	int			dosign;
 	const char *cvt = "0123456789abcdef";
 	int			signvalue = 0;
 	char		convert[64];
 	int			vallen = 0;
-	int			padlen = 0;		/* amount to pad */
+	int			padlen;			/* amount to pad */
 	int			zeropad;		/* extra leading zeroes */
 
 	switch (type)
@@ -947,9 +1110,19 @@ fmtint(int64 value, char type, int forcesign, int leftjust,
 			return;				/* keep compiler quiet */
 	}
 
+	/* disable MSVC warning about applying unary minus to an unsigned value */
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable: 4146)
+#endif
 	/* Handle +/- */
 	if (dosign && adjust_sign((value < 0), forcesign, &signvalue))
-		value = -value;
+		uvalue = -(unsigned long long) value;
+	else
+		uvalue = (unsigned long long) value;
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
 
 	/*
 	 * SUS: the result of converting 0 with an explicit precision of 0 is no
@@ -960,46 +1133,43 @@ fmtint(int64 value, char type, int forcesign, int leftjust,
 	else
 	{
 		/* make integer string */
-		uint64		uvalue = (uint64) value;
-
 		do
 		{
-			convert[vallen++] = cvt[uvalue % base];
+			convert[sizeof(convert) - (++vallen)] = cvt[uvalue % base];
 			uvalue = uvalue / base;
 		} while (uvalue);
 	}
 
 	zeropad = Max(0, precision - vallen);
 
-	adjust_padlen(minlen, vallen + zeropad, leftjust, &padlen);
+	padlen = compute_padlen(minlen, vallen + zeropad, leftjust);
 
-	leading_pad(zpad, &signvalue, &padlen, target);
+	leading_pad(zpad, signvalue, &padlen, target);
 
-	while (zeropad-- > 0)
-		dopr_outch('0', target);
+	if (zeropad > 0)
+		dopr_outchmulti('0', zeropad, target);
 
-	while (vallen > 0)
-		dopr_outch(convert[--vallen], target);
+	dostr(convert + sizeof(convert) - vallen, vallen, target);
 
-	trailing_pad(&padlen, target);
+	trailing_pad(padlen, target);
 }
 
 static void
 fmtchar(int value, int leftjust, int minlen, PrintfTarget *target)
 {
-	int			padlen = 0;		/* amount to pad */
+	int			padlen;			/* amount to pad */
 
-	adjust_padlen(minlen, 1, leftjust, &padlen);
+	padlen = compute_padlen(minlen, 1, leftjust);
 
-	while (padlen > 0)
+	if (padlen > 0)
 	{
-		dopr_outch(' ', target);
-		--padlen;
+		dopr_outchmulti(' ', padlen, target);
+		padlen = 0;
 	}
 
 	dopr_outch(value, target);
 
-	trailing_pad(&padlen, target);
+	trailing_pad(padlen, target);
 }
 
 static void
@@ -1010,10 +1180,10 @@ fmtfloat(double value, char type, int forcesign, int leftjust,
 	int			signvalue = 0;
 	int			prec;
 	int			vallen;
-	char		fmt[32];
+	char		fmt[8];
 	char		convert[1024];
 	int			zeropadlen = 0; /* amount to pad with zeroes */
-	int			padlen = 0;		/* amount to pad with spaces */
+	int			padlen;			/* amount to pad with spaces */
 
 	/*
 	 * We rely on the regular C library's sprintf to do the basic conversion,
@@ -1028,34 +1198,82 @@ fmtfloat(double value, char type, int forcesign, int leftjust,
 	 * bytes and limit requested precision to 350 digits; this should prevent
 	 * buffer overrun even with non-IEEE math.  If the original precision
 	 * request was more than 350, separately pad with zeroes.
+	 *
+	 * We handle infinities and NaNs specially to ensure platform-independent
+	 * output.
 	 */
 	if (precision < 0)			/* cover possible overflow of "accum" */
 		precision = 0;
 	prec = Min(precision, 350);
 
-	if (pointflag)
+	if (isnan(value))
 	{
-		if (sprintf(fmt, "%%.%d%c", prec, type) < 0)
-			goto fail;
-		zeropadlen = precision - prec;
+		strcpy(convert, "NaN");
+		vallen = 3;
+		/* no zero padding, regardless of precision spec */
 	}
-	else if (sprintf(fmt, "%%%c", type) < 0)
-		goto fail;
+	else
+	{
+		/*
+		 * Handle sign (NaNs have no sign, so we don't do this in the case
+		 * above).  "value < 0.0" will not be true for IEEE minus zero, so we
+		 * detect that by looking for the case where value equals 0.0
+		 * according to == but not according to memcmp.
+		 */
+		static const double dzero = 0.0;
 
-	if (!isnan(value) && adjust_sign((value < 0), forcesign, &signvalue))
-		value = -value;
+		if (adjust_sign((value < 0.0 ||
+						 (value == 0.0 &&
+						  memcmp(&value, &dzero, sizeof(double)) != 0)),
+						forcesign, &signvalue))
+			value = -value;
 
-	vallen = sprintf(convert, fmt, value);
-	if (vallen < 0)
-		goto fail;
+		if (isinf(value))
+		{
+			strcpy(convert, "Infinity");
+			vallen = 8;
+			/* no zero padding, regardless of precision spec */
+		}
+		else if (pointflag)
+		{
+			zeropadlen = precision - prec;
+			fmt[0] = '%';
+			fmt[1] = '.';
+			fmt[2] = '*';
+			fmt[3] = type;
+			fmt[4] = '\0';
+			vallen = sprintf(convert, fmt, prec, value);
+		}
+		else
+		{
+			fmt[0] = '%';
+			fmt[1] = type;
+			fmt[2] = '\0';
+			vallen = sprintf(convert, fmt, value);
+		}
+		if (vallen < 0)
+			goto fail;
 
-	/* If it's infinity or NaN, forget about doing any zero-padding */
-	if (zeropadlen > 0 && !isdigit((unsigned char) convert[vallen - 1]))
-		zeropadlen = 0;
+		/*
+		 * Windows, alone among our supported platforms, likes to emit
+		 * three-digit exponent fields even when two digits would do.  Hack
+		 * such results to look like the way everyone else does it.
+		 */
+#ifdef WIN32
+		if (vallen >= 6 &&
+			convert[vallen - 5] == 'e' &&
+			convert[vallen - 3] == '0')
+		{
+			convert[vallen - 3] = convert[vallen - 2];
+			convert[vallen - 2] = convert[vallen - 1];
+			vallen--;
+		}
+#endif
+	}
 
-	adjust_padlen(minlen, vallen + zeropadlen, leftjust, &padlen);
+	padlen = compute_padlen(minlen, vallen + zeropadlen, leftjust);
 
-	leading_pad(zpad, &signvalue, &padlen, target);
+	leading_pad(zpad, signvalue, &padlen, target);
 
 	if (zeropadlen > 0)
 	{
@@ -1066,18 +1284,16 @@ fmtfloat(double value, char type, int forcesign, int leftjust,
 			epos = strrchr(convert, 'E');
 		if (epos)
 		{
-			/* pad after exponent */
+			/* pad before exponent */
 			dostr(convert, epos - convert, target);
-			while (zeropadlen-- > 0)
-				dopr_outch('0', target);
+			dopr_outchmulti('0', zeropadlen, target);
 			dostr(epos, vallen - (epos - convert), target);
 		}
 		else
 		{
 			/* no exponent, pad after the digits */
 			dostr(convert, vallen, target);
-			while (zeropadlen-- > 0)
-				dopr_outch('0', target);
+			dopr_outchmulti('0', zeropadlen, target);
 		}
 	}
 	else
@@ -1086,16 +1302,124 @@ fmtfloat(double value, char type, int forcesign, int leftjust,
 		dostr(convert, vallen, target);
 	}
 
-	trailing_pad(&padlen, target);
+	trailing_pad(padlen, target);
 	return;
 
 fail:
 	target->failed = true;
 }
 
+/*
+ * Nonstandard entry point to print a double value efficiently.
+ *
+ * This is approximately equivalent to strfromd(), but has an API more
+ * adapted to what float8out() wants.  The behavior is like snprintf()
+ * with a format of "%.ng", where n is the specified precision.
+ * However, the target buffer must be nonempty (i.e. count > 0), and
+ * the precision is silently bounded to a sane range.
+ */
+int
+pg_strfromd(char *str, size_t count, int precision, double value)
+{
+	PrintfTarget target;
+	int			signvalue = 0;
+	int			vallen;
+	char		fmt[8];
+	char		convert[64];
+
+	/* Set up the target like pg_snprintf, but require nonempty buffer */
+	Assert(count > 0);
+	target.bufstart = target.bufptr = str;
+	target.bufend = str + count - 1;
+	target.stream = NULL;
+	target.nchars = 0;
+	target.failed = false;
+
+	/*
+	 * We bound precision to a reasonable range; the combination of this and
+	 * the knowledge that we're using "g" format without padding allows the
+	 * convert[] buffer to be reasonably small.
+	 */
+	if (precision < 1)
+		precision = 1;
+	else if (precision > 32)
+		precision = 32;
+
+	/*
+	 * The rest is just an inlined version of the fmtfloat() logic above,
+	 * simplified using the knowledge that no padding is wanted.
+	 */
+	if (isnan(value))
+	{
+		strcpy(convert, "NaN");
+		vallen = 3;
+	}
+	else
+	{
+		static const double dzero = 0.0;
+
+		if (value < 0.0 ||
+			(value == 0.0 &&
+			 memcmp(&value, &dzero, sizeof(double)) != 0))
+		{
+			signvalue = '-';
+			value = -value;
+		}
+
+		if (isinf(value))
+		{
+			strcpy(convert, "Infinity");
+			vallen = 8;
+		}
+		else
+		{
+			fmt[0] = '%';
+			fmt[1] = '.';
+			fmt[2] = '*';
+			fmt[3] = 'g';
+			fmt[4] = '\0';
+			vallen = sprintf(convert, fmt, precision, value);
+			if (vallen < 0)
+			{
+				target.failed = true;
+				goto fail;
+			}
+
+#ifdef WIN32
+			if (vallen >= 6 &&
+				convert[vallen - 5] == 'e' &&
+				convert[vallen - 3] == '0')
+			{
+				convert[vallen - 3] = convert[vallen - 2];
+				convert[vallen - 2] = convert[vallen - 1];
+				vallen--;
+			}
+#endif
+		}
+	}
+
+	if (signvalue)
+		dopr_outch(signvalue, &target);
+
+	dostr(convert, vallen, &target);
+
+fail:
+	*(target.bufptr) = '\0';
+	return target.failed ? -1 : (target.bufptr - target.bufstart
+								 + target.nchars);
+}
+
+
 static void
 dostr(const char *str, int slen, PrintfTarget *target)
 {
+	/* fast path for common case of slen == 1 */
+	if (slen == 1)
+	{
+		dopr_outch(*str, target);
+		return;
+	}
+
 	while (slen > 0)
 	{
 		int			avail;
@@ -1108,7 +1432,10 @@ dostr(const char *str, int slen, PrintfTarget *target)
 		{
 			/* buffer full, can we dump to stream? */
 			if (target->stream == NULL)
-				return;			/* no, lose the data */
+			{
+				target->nchars += slen; /* no, lose the data */
+				return;
+			}
 			flushbuffer(target);
 			continue;
 		}
@@ -1127,12 +1454,51 @@ dopr_outch(int c, PrintfTarget *target)
 	{
 		/* buffer full, can we dump to stream? */
 		if (target->stream == NULL)
-			return;				/* no, lose the data */
+		{
+			target->nchars++;	/* no, lose the data */
+			return;
+		}
 		flushbuffer(target);
 	}
 	*(target->bufptr++) = c;
 }
 
+static void
+dopr_outchmulti(int c, int slen, PrintfTarget *target)
+{
+	/* fast path for common case of slen == 1 */
+	if (slen == 1)
+	{
+		dopr_outch(c, target);
+		return;
+	}
+
+	while (slen > 0)
+	{
+		int			avail;
+
+		if (target->bufend != NULL)
+			avail = target->bufend - target->bufptr;
+		else
+			avail = slen;
+		if (avail <= 0)
+		{
+			/* buffer full, can we dump to stream? */
+			if (target->stream == NULL)
+			{
+				target->nchars += slen; /* no, lose the data */
+				return;
+			}
+			flushbuffer(target);
+			continue;
+		}
+		avail = Min(avail, slen);
+		memset(target->bufptr, c, avail);
+		target->bufptr += avail;
+		slen -= avail;
+	}
+}
+
 
 static int
 adjust_sign(int is_negative, int forcesign, int *signvalue)
@@ -1148,42 +1514,48 @@ adjust_sign(int is_negative, int forcesign, int *signvalue)
 }
 
 
-static void
-adjust_padlen(int minlen, int vallen, int leftjust, int *padlen)
+static int
+compute_padlen(int minlen, int vallen, int leftjust)
 {
-	*padlen = minlen - vallen;
-	if (*padlen < 0)
-		*padlen = 0;
+	int			padlen;
+
+	padlen = minlen - vallen;
+	if (padlen < 0)
+		padlen = 0;
 	if (leftjust)
-		*padlen = -(*padlen);
+		padlen = -padlen;
+	return padlen;
 }
 
 
 static void
-leading_pad(int zpad, int *signvalue, int *padlen, PrintfTarget *target)
+leading_pad(int zpad, int signvalue, int *padlen, PrintfTarget *target)
 {
+	int			maxpad;
+
 	if (*padlen > 0 && zpad)
 	{
-		if (*signvalue)
+		if (signvalue)
 		{
-			dopr_outch(*signvalue, target);
+			dopr_outch(signvalue, target);
 			--(*padlen);
-			*signvalue = 0;
+			signvalue = 0;
 		}
-		while (*padlen > 0)
+		if (*padlen > 0)
 		{
-			dopr_outch(zpad, target);
-			--(*padlen);
+			dopr_outchmulti(zpad, *padlen, target);
+			*padlen = 0;
 		}
 	}
-	while (*padlen > (*signvalue != 0))
+	maxpad = (signvalue != 0);
+	if (*padlen > maxpad)
 	{
-		dopr_outch(' ', target);
-		--(*padlen);
+		dopr_outchmulti(' ', *padlen - maxpad, target);
+		*padlen = maxpad;
 	}
-	if (*signvalue)
+	if (signvalue)
 	{
-		dopr_outch(*signvalue, target);
+		dopr_outch(signvalue, target);
 		if (*padlen > 0)
 			--(*padlen);
 		else if (*padlen < 0)
@@ -1193,11 +1565,8 @@ leading_pad(int zpad, int *signvalue, int *padlen, PrintfTarget *target)
 
 
 static void
-trailing_pad(int *padlen, PrintfTarget *target)
+trailing_pad(int padlen, PrintfTarget *target)
 {
-	while (*padlen < 0)
-	{
-		dopr_outch(' ', target);
-		++(*padlen);
-	}
+	if (padlen < 0)
+		dopr_outchmulti(' ', -padlen, target);
 }
