@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2015	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -25,6 +25,11 @@
 #include "pool.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
+#include "utils/pool_signal.h"
+#include "utils/pool_ipc.h"
+#include "utils/ps_status.h"
+
+#include "pcp/pcp_worker.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -262,8 +267,8 @@ start_pcp_command_processor_process(int port)
 
 	if (pid == 0)				/* child */
 	{
-		/* Set the process type variable */
-		processType = PT_PCP_WORKER;
+		SetProcessGlobalVaraibles(PT_PCP_WORKER);
+
 		on_exit_reset();
 		/* Close the listen sockets sockets */
 		close(pcp_unix_fd);
