@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -29,6 +29,7 @@
 
 #include "pool.h"
 #include "protocol/pool_proto_modules.h"
+#include "protocol/pool_process_query.h"
 #include "parser/pg_config_manual.h"
 #include "parser/pool_string.h"
 #include "pool_config.h"
@@ -44,9 +45,6 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 static int	foward_command_complete(POOL_CONNECTION * frontend, char *packet, int packetlen);
 static int	foward_empty_query(POOL_CONNECTION * frontend, char *packet, int packetlen);
 static int	foward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, int packetlen);
-static void process_clear_cache(POOL_CONNECTION_POOL * backend);
-static void clear_query_cache(void);
-static bool check_alter_role_statement(AlterRoleStmt *stmt);
 
 POOL_STATUS
 CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool command_complete)
@@ -123,11 +121,11 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 	}
 
 	/*
-	 * Otherwise just read from master node.
+	 * Otherwise just read from main node.
 	 */
 	else
 	{
-		con = MASTER(backend);
+		con = MAIN(backend);
 
 		if (pool_read(con, &len, sizeof(len)) < 0)
 			return POOL_END;
@@ -143,36 +141,6 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 		memcpy(p1, p, len);
 	}
 
-	/*
-	 * Check if a transaction start command was issued by a multi statement
-	 * query by looking at the command tag. We cannot look into the parse tree
-	 * because our parse tree is only the first query in the multi statement.
-	 */
-	if (SL_MODE && session_context->query_context &&
-		session_context->query_context->is_multi_statement)
-	{
-		if (!strcmp(p1, "BEGIN"))
-		{
-			/*
-			 * If the query was a transaction starting command, remember it
-			 * until it gets committed or roll backed.
-			 */
-			elog(DEBUG1, "Call set_tx_started_by_multi_statement_query() in CommandComplete");
-			set_tx_started_by_multi_statement_query();
-		}
-		else if (!strcmp(p1, "COMMIT") || !strcmp(p1, "ROLLBACK"))
-		{
-			/*
-			 * It is possible that the multi statement query included both
-			 * BEGIN and COMMIT/ROLLBACK, or just COMMIT/ROLLBACK command.  In
-			 * this case we forget that a transaction was started by multi
-			 * statement query.
-			 */
-			elog(DEBUG1, "Call unset_tx_started_by_multi_statement_query() in CommandComplete");
-			unset_tx_started_by_multi_statement_query();
-		}
-	}
-
 	/*
 	 * If operated in streaming replication mode and extended query mode, just
 	 * forward the packet to frontend and we are done. Otherwise, we need to
@@ -212,10 +180,10 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 		}
 
 		/*
-		 * If we are doing extended query, register the SELECT result to temp
-		 * query cache now.
+		 * If we are in streaming replication mode and we are doing extended
+		 * query, register query cache now.
 		 */
-		if (pool_is_doing_extended_query_message())
+		if (SL_MODE && pool_is_doing_extended_query_message())
 		{
 			char	   *query;
 			Node	   *node;
@@ -223,12 +191,12 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 
 			if (session_context->query_context == NULL)
 			{
-				elog(WARNING, "expected query_contex is NULL");
+				elog(WARNING, "expected query_contex is not NULL");
 				return POOL_END;
 			}
 			query = session_context->query_context->query_w_hex;
 			node = pool_get_parse_tree();
-			state = TSTATE(backend, MASTER_NODE_ID);
+			state = TSTATE(backend, MAIN_NODE_ID);
 			pool_handle_query_cache(backend, query, node, state);
 		}
 	}
@@ -367,35 +335,11 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 		{
 			/* Commit ongoing CRETAE/DROP temp table status */
 			pool_temp_tables_commit_pending();			
-
-			/* Forget a transaction was started by multi statement query */
-			unset_tx_started_by_multi_statement_query();
-
-			/*
-			 * Query cache enabled and REVOKE was issued in this transaction?
-			 */
-			if (pool_config->memory_cache_enabled &&
-				query_cache_disabled_tx())
-			{
-				/*
-				 * Clear all the query cache.
-				 */
-				clear_query_cache();
-
-				/* Unset query cache disabled in this transaction */
-				unset_query_cache_disabled_tx();
-			}
 		}
 		else if (stmt->kind == TRANS_STMT_ROLLBACK)
 		{
 			/* Remove ongoing CRETAE/DROP temp table status */
 			pool_temp_tables_remove_pending();
-
-			/* Forget a transaction was started by multi statement query */
-			unset_tx_started_by_multi_statement_query();
-
-			/* Unset query cache disabled in this transaction */
-			unset_query_cache_disabled_tx();
 		}
 	}
 	else if (IsA(node, CreateStmt))
@@ -406,7 +350,7 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 		/* Is this a temporary table? */
 		if (stmt->relation->relpersistence == 't')
 		{
-			if (TSTATE(backend, MASTER_NODE_ID ) == 'T')	/* Are we inside a transaction? */
+			if (TSTATE(backend, MAIN_NODE_ID ) == 'T')	/* Are we inside a transaction? */
 			{
 				state = TEMP_TABLE_CREATING;
 			}
@@ -431,7 +375,7 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 			ListCell   *cell;
 			ListCell   *next;
 
-			if (TSTATE(backend, MASTER_NODE_ID ) == 'T')	/* Are we inside a transaction? */
+			if (TSTATE(backend, MAIN_NODE_ID ) == 'T')	/* Are we inside a transaction? */
 			{
 				state = TEMP_TABLE_DROPPING;
 			}
@@ -446,71 +390,14 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 				ereport(DEBUG1,
 						(errmsg("Dropping temp table: %s", tablename)));
 				pool_temp_tables_delete(tablename, state);
-				next = lnext(cell);
+
+				if (!session_context->temp_tables)
+					return;
+
+				next = lnext(session_context->temp_tables, cell);
 			}
 		}
 	}
-	else if (IsA(node, VariableSetStmt))
-	{
-		VariableSetStmt *stmt = (VariableSetStmt *) node;
-
-		/* SET ROLE or SESSION AUTHORIZATION command? */
-		if (stmt->kind == VAR_SET_VALUE &&
-			(!strcmp(stmt->name, "role") ||
-			 !strcmp(stmt->name, "session_authorization")))
-			/* disable query cache in this session */
-			set_query_cache_disabled();
-	}
-	else if (IsA(node, GrantStmt))
-	{
-		GrantStmt *stmt = (GrantStmt *) node;
-
-		/* REVOKE? */
-		if (stmt->is_grant)
-			return;
-
-		/* Clear query cache */
-		process_clear_cache(backend);
-	}
-	else if (IsA(node, AlterTableStmt) || IsA(node, AlterDatabaseStmt) ||
-			 IsA(node, AlterDatabaseSetStmt))
-	{
-		/* Clear query cache */
-		process_clear_cache(backend);
-	}
-	else if (IsA(node, AlterRoleStmt))
-	{
-		if (check_alter_role_statement(castNode(AlterRoleStmt, node)))
-		{
-			/* Clear query cache */
-			process_clear_cache(backend);
-		}
-	}
-}
-
-/*
- * Check whether the ALTER ROLE statement needs query cache invalidation.
- * stmt must be AlterRoleStmt.
- */
-static bool
-check_alter_role_statement(AlterRoleStmt *stmt)
-{
-	ListCell	*l;
-
-	foreach(l, stmt->options)
-	{
-		DefElem	*elm = (DefElem *) lfirst(l);
-
-		/*
-		 * We want to detect other than ALTER ROLE foo WITH PASSWORD or
-		 * WITH CONNECTION LIMIT case.  It does not change any privilege of the
-		 * role.
-		 */
-		if (strcmp(elm->defname, "password") &&
-			strcmp(elm->defname, "connectionlimit"))
-			return true;
-	}
-	return false;
 }
 
 /*
@@ -553,14 +440,14 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 	rows = extract_ntuples(packet);
 
 	/*
-	 * Save number of affected tuples of master node.
+	 * Save number of affected tuples of main node.
 	 */
-	session_context->ntuples[MASTER_NODE_ID] = rows;
+	session_context->ntuples[MAIN_NODE_ID] = rows;
 
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
-		if (!IS_MASTER_NODE_ID(i))
+		if (!IS_MAIN_NODE_ID(i))
 		{
 			if (!VALID_BACKEND(i))
 			{
@@ -581,7 +468,7 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 			{
 				ereport(DEBUG1,
 						(errmsg("processing command complete"),
-						 errdetail("length does not match between backends master(%d) %d th backend(%d)",
+						 errdetail("length does not match between backends main(%d) %d th backend(%d)",
 								   len, i, packetlen)));
 			}
 
@@ -613,7 +500,7 @@ static POOL_STATUS handle_mismatch_tuples(POOL_CONNECTION * frontend, POOL_CONNE
 		string_append_char(msg, "\"");
 		pool_send_error_message(frontend, MAJOR(backend),
 								"XX001", msg->data, "",
-								"check data consistency between master and other db node", __FILE__, __LINE__);
+								"check data consistency between main and other db node", __FILE__, __LINE__);
 		ereport(LOG,
 				(errmsg("%s", msg->data)));
 		free_string(msg);
@@ -685,103 +572,3 @@ foward_packet_to_frontend(POOL_CONNECTION * frontend, char kind, char *packet, i
 
 	return 0;
 }
-
-/*
- * Process statements that need clearing query cache
- */
-static void
-process_clear_cache(POOL_CONNECTION_POOL * backend)
-{
-	/* Query cache enabled? */
-	if (!pool_config->memory_cache_enabled)
-		return;
-
-	/* Streaming replication mode? */
-	if (SL_MODE)
-	{
-		/*
-		 * Are we inside a transaction?
-		 */
-		if (TSTATE(backend, MASTER_NODE_ID ) == 'T')
-		{
-			/*
-			 * Disable query cache in this transaction.
-			 * All query cache will be cleared at commit.
-			 */
-			set_query_cache_disabled_tx();
-		}
-		else if (TSTATE(backend, MASTER_NODE_ID ) == 'I')	/* outside transaction */
-		{
-			/*
-			 * Clear all the query cache.
-			 */
-			clear_query_cache();
-		}
-	}
-	else
-	{
-		/*
-		 * Are we inside a transaction?
-		 */
-		if (TSTATE(backend, MASTER_NODE_ID ) == 'T')
-		{
-			/* Inside user started transaction? */
-			if (!INTERNAL_TRANSACTION_STARTED(backend, MASTER_NODE_ID))
-			{
-				/*
-				 * Disable query cache in this transaction.
-				 * All query cache will be cleared at commit.
-				 */
-				set_query_cache_disabled_tx();
-			}
-			else
-			{
-				/*
-				 * Clear all the query cache.
-				 */
-				clear_query_cache();
-			}
-		}
-		else if (TSTATE(backend, MASTER_NODE_ID ) == 'I')	/* outside transaction */
-		{
-			/*
-			 * Clear all the query cache.
-			 */
-			clear_query_cache();
-		}
-	}
-}
-
-/*
- * Clear query cache on shmem or memcached
- */
-static
-void clear_query_cache(void)
-{
-	/*
-	 * Clear all the shared memory cache and oid maps.
-	 */
-	if (pool_is_shmem_cache())
-	{
-		pool_clear_memory_cache();
-		ereport(LOG,
-				(errmsg("all query cache in shared memory deleted")));
-	}
-	else
-#ifdef USE_MEMCACHED
-	{
-		/*
-		 * Clear all the memcached cache and oid maps.
-		 */
-		delete_all_cache_on_memcached();
-		pool_discard_oid_maps();
-		ereport(LOG,
-				(errmsg("all query cache in memcached deleted")));
-	}
-#else
-	{
-		ereport(WARNING,
-				(errmsg("failed to clear cache on memcached, memcached support is not enabled")));
-	}
-#endif
-}
