@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2021	PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -49,20 +49,22 @@
 #endif
 
 #include "pool.h"
+#include "pool_config.h"
+
 #include "utils/palloc.h"
 #include "utils/memutils.h"
 #include "utils/elog.h"
+#include "utils/pool_ip.h"
+#include "utils/ps_status.h"
+#include "utils/pool_stream.h"
 
 #include "context/pool_process_context.h"
 #include "context/pool_session_context.h"
-#include "pool_config.h"
-#include "utils/pool_ip.h"
+#include "protocol/pool_process_query.h"
+#include "protocol/pool_pg_utils.h"
+#include "main/pool_internal_comms.h"
 #include "auth/md5.h"
 #include "auth/pool_hba.h"
-#include "utils/pool_stream.h"
-
-#include "watchdog/wd_ipc_commands.h"
-#include "watchdog/watchdog.h"
 
 static POOL_CONNECTION_POOL_SLOT * slots[MAX_NUM_BACKENDS];
 static volatile sig_atomic_t reload_config_request = 0;
@@ -76,7 +78,6 @@ static unsigned long long int text_to_lsn(char *text);
 static RETSIGTYPE my_signal_handler(int sig);
 static RETSIGTYPE reload_config_handler(int sig);
 static void reload_config(void);
-static void sr_check_will_die(int code, Datum arg);
 
 #define CHECK_REQUEST \
 	do { \
@@ -95,8 +96,6 @@ static void sr_check_will_die(int code, Datum arg);
 #define PG10_SERVER_VERSION	100000	/* PostgreSQL 10 server version num */
 #define PG91_SERVER_VERSION	90100	/* PostgreSQL 9.1 server version num */
 
-static	volatile bool follow_primary_lock_acquired;
-
 /*
 * worker child main loop
 */
@@ -113,11 +112,6 @@ do_worker_child(void)
 	init_ps_display("", "", "", "");
 	set_ps_display("worker process", false);
 
-	/*
-	 * install the call back for preparation of exit
-	 */
-	on_system_exit(sr_check_will_die, (Datum) NULL);
-
 	/* set up signal handlers */
 	signal(SIGALRM, SIG_DFL);
 	signal(SIGTERM, my_signal_handler);
@@ -158,15 +152,8 @@ do_worker_child(void)
 
 	for (;;)
 	{
-		WD_STATES	wd_status;
-
 		MemoryContextSwitchTo(WorkerMemoryContext);
 		MemoryContextResetAndDeleteChildren(WorkerMemoryContext);
-		/*
-		 * Since WorkerMemoryContext is used for "slots", we need to clear it
-		 * so that new slots are allocated later on.
-		 */
-		memset(slots, 0, sizeof(slots));
 
 		CHECK_REQUEST;
 
@@ -175,92 +162,52 @@ do_worker_child(void)
 			sleep(30);
 		}
 
-		/*
-		 * Get watchdog status if watchdog is enabled.
-		 */
-		if (pool_config->use_watchdog)
-		{
-			wd_status = get_watchdog_local_node_state();
-			ereport(DEBUG1,
-					(errmsg("watchdog status: %d", wd_status)));
-		}
-
 		/*
 		 * If streaming replication mode, do time lag checking
-		 * Also skip if failover/failback is ongoing.
 		 */
-		if (pool_config->sr_check_period > 0 && STREAM &&
-			Req_info->switching == false)
+
+		if (pool_config->sr_check_period > 0 && STREAM)
 		{
-			/*
-			 * Acquire follow primary lock. If fail to acquire lock, try again.
-			 */
-			follow_primary_lock_acquired = false;
-
-			if (pool_acquire_follow_primary_lock(false) == true)
+			establish_persistent_connection();
+			PG_TRY();
 			{
-				follow_primary_lock_acquired = true;
+				POOL_NODE_STATUS *node_status;
+				int			i;
 
-				establish_persistent_connection();
-				PG_TRY();
+				/* Do replication time lag checking */
+				check_replication_time_lag();
+
+				/* Check node status */
+				node_status = verify_backend_node_status(slots);
+				for (i = 0; i < NUM_BACKENDS; i++)
 				{
-					POOL_NODE_STATUS *node_status;
-					int			i;
+					ereport(DEBUG1,
+							(errmsg("node status[%d]: %d", i, node_status[i])));
 
-					/* Do replication time lag checking */
-					check_replication_time_lag();
-
-					/* Check node status */
-					node_status = verify_backend_node_status(slots);
-
-
-					for (i = 0; i < NUM_BACKENDS; i++)
+					if (node_status[i] == POOL_NODE_STATUS_INVALID)
 					{
-						ereport(DEBUG1,
-								(errmsg("node status[%d]: %d", i, node_status[i])));
+						int			n;
 
-						if (node_status[i] == POOL_NODE_STATUS_INVALID)
+						ereport(LOG,
+								(errmsg("pgpool_worker_child: invalid node found %d", i)));
+						if (pool_config->detach_false_primary)
 						{
-							int			n;
-
-							ereport(LOG,
-									(errmsg("pgpool_worker_child: invalid node found %d", i)));
-							/*
-							 * If detach_false_primary is enabled, send
-							 * degenerate request to detach invalid node.
-							 * This should only happen on leader watchdog node
-							 * and quorum exists if watchdog is enabled. Other
-							 * nodes will be informed by the leader node later
-							 * on.
-							 */
-							if ((pool_config->detach_false_primary && !pool_config->use_watchdog) ||
-								(pool_config->detach_false_primary && pool_config->use_watchdog &&
-								 get_watchdog_quorum_state() >= 0 && wd_status == WD_COORDINATOR))
-							{
-								n = i;
-								degenerate_backend_set(&n, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED);
-							}
+							n = i;
+							degenerate_backend_set(&n, 1, REQ_DETAIL_SWITCHOVER | REQ_DETAIL_CONFIRMED);
 						}
 					}
 				}
-				PG_CATCH();
-				{
-					discard_persistent_connection();
-					pool_release_follow_primary_lock();
-					follow_primary_lock_acquired = false;
-					sleep(pool_config->sr_check_period);
-					PG_RE_THROW();
-				}
-				PG_END_TRY();
-
-				/* Discard persistent connections */
-				discard_persistent_connection();
-				if (follow_primary_lock_acquired)
-				{
-					pool_release_follow_primary_lock();
-					follow_primary_lock_acquired = false;
-				}
 			}
+			PG_CATCH();
+			{
+				discard_persistent_connection();
+				sleep(pool_config->sr_check_period);
+				PG_RE_THROW();
+			}
+			PG_END_TRY();
+
+			/* Discard persistent connections */
+			discard_persistent_connection();
 		}
 		sleep(pool_config->sr_check_period);
 	}
@@ -291,7 +238,7 @@ establish_persistent_connection(void)
 															 bkinfo->backend_port,
 															 pool_config->sr_check_database,
 															 pool_config->sr_check_user,
-															 password ? password : "", false);
+															 password ? password : "", true);
 		}
 	}
 
@@ -358,20 +305,6 @@ check_replication_time_lag(void)
 		return;
 	}
 
-	if (!VALID_BACKEND(REAL_PRIMARY_NODE_ID))
-	{
-		/*
-		 * No need to check replication delay if primary is down. This could
-		 * happen if ALWAYS_PRIMARY flag is on because REAL_PRIMARY_NODE_ID
-		 * macro returns the node id which ALWAYS_PRIMARY flag is set to.  If
-		 * we do not check this, subsequent test (i == PRIMARY_NODE_ID) in the
-		 * for loop below will return unexpected result because
-		 * PRIMARY_NODE_ID macro returns MAIN_NODE_ID, which could be a
-		 * standby server.
-		 */
-		return;
-	}
-
 	/*
 	 * Register a error context callback to throw proper context message
 	 */
@@ -400,7 +333,7 @@ check_replication_time_lag(void)
 
 		if (server_version[i] == 0)
 		{
-			query = "SELECT pg_catalog.current_setting('server_version_num')";
+			query = "SELECT current_setting('server_version_num')";
 
 			/*
 			 * Get backend server version. If the query fails, keep previous
@@ -418,21 +351,21 @@ check_replication_time_lag(void)
 		if (PRIMARY_NODE_ID == i)
 		{
 			if (server_version[i] >= PG10_SERVER_VERSION)
-				query = "SELECT pg_catalog.pg_current_wal_lsn()";
+				query = "SELECT pg_current_wal_lsn()";
 			else
-				query = "SELECT pg_catalog.pg_current_xlog_location()";
+				query = "SELECT pg_current_xlog_location()";
 
 			if (server_version[i] == PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, '' AS sync_state FROM pg_catalog.pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, '' AS sync_state FROM pg_stat_replication";
 			else if (server_version[i] > PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, sync_state FROM pg_catalog.pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, sync_state FROM pg_stat_replication";
 		}
 		else
 		{
 			if (server_version[i] >= PG10_SERVER_VERSION)
-				query = "SELECT pg_catalog.pg_last_wal_replay_lsn()";
+				query = "SELECT pg_last_wal_replay_lsn()";
 			else
-				query = "SELECT pg_catalog.pg_last_xlog_replay_location()";
+				query = "SELECT pg_last_xlog_replay_location()";
 
 			active_standby_node++;
 		}
@@ -666,11 +599,3 @@ get_query_result(POOL_CONNECTION_POOL_SLOT * *slots, int backend_id, char *query
 	sts = 0;
 	return sts;
 }
-
-static void
-sr_check_will_die(int code, Datum arg)
-{
-	if (follow_primary_lock_acquired)
-		pool_release_follow_primary_lock();
-
-}
