@@ -38,7 +38,7 @@ function install_pgpool
 
 	test -d $log || mkdir $log
         
-	make install WATCHDOG_DEBUG=1 -C $dir/../../ -e prefix=${PGPOOL_PATH} >& regression.log 2>&1
+	make install HEALTHCHECK_DEBUG=1 WATCHDOG_DEBUG=1 -C $dir/../../ -e prefix=${PGPOOL_PATH} >& regression.log 2>&1
 
 	if [ $? != 0 ];then
 	    echo "make install failed"
@@ -92,12 +92,7 @@ function export_env_vars
  	fi
 	
 	echo "using pgpool-II at "$PGPOOL_PATH
-
-	export PGPOOL_VERSION=`$PGPOOL_PATH/bin/pgpool --version 2>&1`
-
 	export PGPOOL_INSTALL_DIR=$PGPOOL_PATH
-	# where to look for pgpool.conf.sample files.
-	export PGPOOLDIR=${PGPOOLDIR:-"$PGPOOL_INSTALL_DIR/etc"}
 
 	PGPOOLLIB=${PGPOOL_INSTALL_DIR}/lib
 	if [ -z "$LD_LIBRARY_PATH" ];then
@@ -112,17 +107,13 @@ function export_env_vars
 	export PGBENCH_PATH=$PGBENCH_PATH
 	export PGSOCKET_DIR=$PGSOCKET_DIR
 	export PGVERSION=`$PGBIN/initdb -V|awk '{print $3}'|sed 's/\..*//'`
-	export LANG=C
-
-	export ENABLE_TEST=true
 }
 function print_info
 {
 	echo ${CBLUE}"*************************"${CNORM}
 
 	echo "REGRESSION MODE          : "${CBLUE}$MODE${CNORM}
-	echo "Pgpool-II version        : "${CBLUE}$PGPOOL_VERSION${CNORM}
-	echo "Pgpool-II install path   : "${CBLUE}$PGPOOL_PATH${CNORM}
+	echo "PGPOOL-II                : "${CBLUE}$PGPOOL_PATH${CNORM}
 	echo "PostgreSQL bin           : "${CBLUE}$PGBIN${CNORM}
 	echo "PostgreSQL Major version : "${CBLUE}$PGVERSION${CNORM}
 	echo "pgbench                  : "${CBLUE}$PGBENCH_PATH${CNORM}
@@ -172,11 +163,11 @@ if [ "$MODE" = "install" ]; then
 
 elif [ "$MODE" = "noinstall" ]; then
 	echo not installing pgpool for the tests ...
-	if [[ -n "$PGPOOL_INSTALL_DIR" ]]; then
-		PGPOOL_PATH=$PGPOOL_INSTALL_DIR
+	if [[ -n "$PGPOOL_INSTALL_PATH" ]]; then
+		PGPOOL_PATH=$PGPOOL_INSTALL_PATH
 	fi
-        export PGPOOL_SETUP=$PGPOOL_PATH/bin/pgpool_setup
-        export WATCHDOG_SETUP=$PGPOOL_PATH/bin/watchdog_setup
+        export PGPOOL_SETUP=$dir/../pgpool_setup
+        export WATCHDOG_SETUP=$dir/../watchdog_setup
 else
 	echo $MODE : Invalid mode
 	exit -1
@@ -202,25 +193,10 @@ fi
 for i in $dirs
 do
 	cd $i
-
-	# skip the test if there's no test.sh
-	if [ ! -f test.sh ];then
-		cd ..
-		continue;
-	fi
-
 	echo -n "testing $i..."
 	clean_all
 	timeout $TIMEOUT ./test.sh > $log/$i 2>&1
 	rtn=$?
-
-	check_segfault
-	if [ $? -eq 0 ];then
-		echo "fail: Segmentation fault detected" >> $log/$i
-		segfault=1
-		rtn=1
-	fi
-
 	if [ $rtn = 0 ];then
 		echo ${CGREEN}"ok."${CNORM}
 		ok=`expr $ok + 1`
@@ -228,11 +204,7 @@ do
 		echo "timeout."
 		timeout=`expr $timeout + 1`
 	else
-		if [ "$segfault" = "1" ];then
-			echo ${CRED}"segfault."${CNORM}
-		else
-			echo ${CRED}"failed."${CNORM}
-		fi
+		echo ${CRED}"failed."${CNORM}
 		fail=`expr $fail + 1`
 	fi
 
@@ -240,6 +212,6 @@ do
 
 done
 
-total=`expr $ok + $fail + $timeout`
+total=`expr $ok + $fail`
 
 echo "out of $total ok:$ok failed:$fail timeout:$timeout"
