@@ -34,6 +34,7 @@
 #include "utils/elog.h"
 #include "pool_config.h"
 #include "watchdog/wd_utils.h"
+#include "utils/pool_signal.h"
 
 #define WD_MAX_PING_RESULT 256
 
@@ -133,7 +134,8 @@ wd_issue_ping_command(char *hostname, int *outfd)
 	{
 		/* CHILD */
 		on_exit_reset();
-		processType = PT_WATCHDOG_UTILITY;
+		SetProcessGlobalVaraibles(PT_WATCHDOG_UTILITY);
+
 		close(STDOUT_FILENO);
 		dup2(pfd[1], STDOUT_FILENO);
 		close(pfd[0]);
